#include <tangle.h>
#include <glib-object.h>
#include <sys/time.h>
#include <stdlib.h>
#include "community_utilities.h"
#include "../jammo.h"
#include "../../meam/jammo-sample.h"
#include <sys/types.h>
#include <dirent.h>
#include "string.h"
#include <time.h>


/*
*Function computes the length of the click.
*If the length of the click is more than 1 seconds, function to rename the item is called.
*Otherwise a function to play the appropriate music piece is called. 
*/

void song_name_press_action(ClutterActor* loopName){

	static gboolean clicked = FALSE;
	static struct timeval timePressed;
	struct timeval timeReleased;

	
	clicked = !clicked;

	if(clicked){
		gettimeofday(&timePressed, NULL);
		}
	
	else{	
		gettimeofday(&timeReleased, NULL);
		
		if(timeReleased.tv_sec-timePressed.tv_sec > 1){
	
			rename_loop(loopName);	
		} 
		else if((timeReleased.tv_sec-timePressed.tv_sec == 1) && (timeReleased.tv_usec >= timePressed.tv_usec)){
			
			rename_loop(loopName);
		
		}
	
		else{
			play_selected_loop(NULL);
	
		}
	}	
	
}


void rename_loop(ClutterActor *loopName){

	//TODO: use writing tool to rename the loop

}

void play_selected_loop(gchar *loopName){
	
	//TODO: add correct loop, this one is just to test the button
	static JammoSample *sample = NULL;
	
	if(sample == NULL){
		//songName = "/home/teppo/jammo10-jammo-data/data/jammo7-12_all/backingtracks/Sa_LaBamba_110_A.ogg";
		puts("Starting loop");

		if(loopName == NULL){
			sample = jammo_sample_new_from_file("/home/teppo/jammo10-jammo-data/data/jammo7-12_all/backingtracks/Sa_LaBamba_110_A.ogg");
		}else{
			sample = jammo_sample_new_from_file(loopName);
		}
	
		jammo_sample_play(sample);
	}
	
	else{
		puts("loop stop");
		jammo_sample_stop(sample);
		g_object_unref(sample);
		sample = NULL;
	
	}

}

void show_song_label(ClutterActor* songName){

	//TODO: show label

	puts("Showing imaginary song label");
}


void show_music_video(ClutterActor* songName){

	//TODO: play music video
	
	puts("Showing imaginary music video");

}

time_type get_current_time(){

	time_t now;
	time_type current_time;
	struct tm *ts;
	gchar temp[80];

	//to get current time and date
	now = time(NULL);

	//to format the time, "dd.mm.yy hh:mm"
	ts = localtime(&now);
	strftime(temp, 18, "%d.%m.%Y %H:%M", ts);
	strcpy(current_time.time, temp);

	return current_time;
}

void clear_container(ClutterContainer *listToClear){
	
	int i = 0;
	GList *listItems;
	
	if(listToClear != NULL){
	 	
		listItems = clutter_container_get_children(listToClear); 
		
		if(listItems != NULL){
		
			do{			
				if(i > 0)
					listItems = listItems->next;
								
				clutter_container_remove_actor(CLUTTER_CONTAINER(listToClear), CLUTTER_ACTOR(listItems->data));
				i++;
	
			}while(listItems->next != NULL);
		
		g_list_free(listItems);

		}

	}
	

	listToClear = 0;	

}

gboolean add_images_from_directory(const char* path, ClutterContainer *list, GCallback function, int width, int height){
		
	ClutterActor *button = 0;
	char tempPath[150];
	char result[MAX_FILES_IN_DIR][MAX_FILE_NAME_LEN];
	int numberOfImages;
	
	numberOfImages = community_open_and_arrange_dir(path, ".png", result);	
	
	for(int i = 0; i < numberOfImages; i++){
		strcpy(tempPath, path);
		strcat(tempPath, result[i]);
		add_single_button(button, list, tempPath, width, height, function);
	
	}
	
  	return TRUE;

}	

gboolean add_single_button(ClutterActor *button, ClutterContainer *list, char* path, int width, int height, GCallback function){

	button = tangle_button_new_with_background_actor(clutter_texture_new_from_file(path, NULL));
	clutter_actor_set_name(button, path);
	clutter_container_add_actor(list, button);
	clutter_actor_set_size(button, height, width);
	g_signal_connect_swapped(button, "clicked", G_CALLBACK(function), button);

	return TRUE;
}



void limit_line_count(TextField *text){

	gchar textField[500];
	PangoLayout *layout;
	int index;
	
	strncpy(textField, clutter_text_get_text(CLUTTER_TEXT(text->text)), 499);
	textField[499] = 0;

	layout = clutter_text_get_layout(CLUTTER_TEXT(text->text));

	while(pango_layout_get_line_count(layout) > text->lineCount){
			index = clutter_text_get_cursor_position(CLUTTER_TEXT(text->text));

			if(index == -1){
				textField[strlen(textField)-1] = 0;
			}else{
				textField[index] = 0;
				strcat(textField, textField+index+1);
				
				clutter_text_set_cursor_position(CLUTTER_TEXT(text->text), index == 0 ? 0 : index-1);
			}
			
			g_signal_handler_disconnect(text->text, text->handler_id);
			clutter_text_set_text(CLUTTER_TEXT(text->text), textField); 
			text->handler_id = g_signal_connect_swapped(text->text, "text-changed", (GCallback)limit_line_count, text);
			layout = clutter_text_get_layout(CLUTTER_TEXT(text->text));
	}
		
}

void clear_messagelist(GList* messages){

	messages = g_list_first(messages);

	message *temp = 0;

	if(messages != NULL){
		
		temp = messages->data;

		if(temp->msg != NULL)
		free(temp->msg);

		if(messages->data != NULL)
		free(messages->data);
	
		while(messages->next != NULL)
		{
			messages = messages->next;
				
			temp = messages->data;

			if(temp->msg != NULL)
			free(temp->msg);
				
			if(messages->data != NULL)
			free(messages->data);				
		}
	
	}



}

gboolean community_get_avatar(Avatar* avatar, guint32 avatar_id){

	int head, hair_color, hair_type, body, instrument;
	char path[150];
	char result[MAX_FILES_IN_DIR][MAX_FILE_NAME_LEN];
	
	printf("%d\n", avatar_id);
	
	head = (avatar_id & 0xF);
	hair_color = (avatar_id & 0xF0) >> 4;
	body = (avatar_id & 0xFF00) >> 8;
	hair_type = (avatar_id & 0xFF0000) >> 16;
	instrument = (avatar_id & 0xFF000000) >> 24;

	if(head != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/heads/");
		
		community_open_and_arrange_dir(path, ".png", result);
		avatar->head = clutter_texture_new_from_file(strcat(path, result[head-1]), NULL);	
		clutter_actor_set_name(avatar->head, path);
	
	}

	if(body != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/bodys/");
		
		community_open_and_arrange_dir(path, ".png", result);
		avatar->body = clutter_texture_new_from_file(strcat(path, result[body-1]), NULL);	
		clutter_actor_set_name(avatar->body, path);
	
	}
	
	if(hair_color != 0 && hair_type != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/hairs/");

		community_open_and_arrange_dir(path, "DIR", result);
		
		strcat(path, result[hair_color-1]);
		strcat(path, "/");
		
		community_open_and_arrange_dir(path, ".png", result);
		
		strcat(path, result[hair_type-1]);
		
		avatar->hair = clutter_texture_new_from_file(path, NULL);
		clutter_actor_set_name(avatar->hair, path);
		
	
	}	
	
	if(instrument != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/INSTRUMENTS/");
		community_open_and_arrange_dir(path, ".png", result);
		avatar->instrument = clutter_texture_new_from_file(strcat(path, result[instrument-1]), NULL);
		clutter_actor_set_name(avatar->instrument, path);
	
	
	}
	
	return TRUE;

}

gboolean community_save_avatar(Avatar* avatar){

	guint32 head = 0, hair_color = 0, hair_type = 0, body = 0, instrument = 0;
	char path[150];
	char result[MAX_FILES_IN_DIR][MAX_FILE_NAME_LEN];
	int i;
	int numberOfImages = 0;
	
	if(avatar->head != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/heads/");

		numberOfImages = community_open_and_arrange_dir(path, ".png", result);
		
		for(i = 0; i < numberOfImages; i++)
			if(strstr(clutter_actor_get_name(avatar->head), result[i]) != 0)
				break;	
				
		head = i+1;		
	}
	
	if(avatar->body != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/bodys/");	
	
		numberOfImages = community_open_and_arrange_dir(path, ".png", result);
		
		for(i = 0; i < numberOfImages; i++)
			if(strstr(clutter_actor_get_name(avatar->body), result[i]) != 0)
				break;	
				
		body = i+1;		
	
	}	
	
	if(avatar->hair != 0){
	
		strcpy(path, "/opt/jammo/communitymenu/AVATAR_PARTS/hairs/");
		
		numberOfImages = community_open_and_arrange_dir(path, "DIR", result);
		
		for(i = 0; i < numberOfImages; i++)
			if(strstr(clutter_actor_get_name(avatar->hair), result[i]) != 0)
				break;	
				
		hair_color = i+1;		
					
		strcpy(path, clutter_actor_get_name(avatar->hair));
		
		char *temp = strrchr(path, '/');
		temp++;
		*temp = 0;
		numberOfImages = community_open_and_arrange_dir(path, ".png", result);
		
		for(i = 0; i < numberOfImages; i++)
			if(strstr(clutter_actor_get_name(avatar->hair), result[i]) != 0)
				break;	
				
		hair_type = i+1;		
	}		
	
	if(avatar->instrument != 0){
		
		strcpy(path, "/opt/jammo/communitymenu/INSTRUMENTS/");
		
		numberOfImages = community_open_and_arrange_dir(path, ".png", result);
		
		for(i = 0; i < numberOfImages; i++)
			if(strstr(clutter_actor_get_name(avatar->instrument), result[i]) != 0)
				break;	
				
		instrument = i+1;
	
	}

	
	guint32 id = 0;
	
	id = head;
	id += (hair_color << 4);
	id += (body << 8);
	id += (hair_type << 16);
	id += (instrument << 24);
	printf("%d\n", id);
	
	return gems_profile_manager_set_avatar_id(id);

}


/*
* This function opens directory, and sorts files in alphabetical order.
* Files must be sorted because file order from opendir() may be different in each device.
*
*parameters: 
* -path is full path of directory to be opened
* -suffix defines which type files will be sorted, other files are ignored
*   (if suffix is "DIR", only directories are listed)
* -result is an array of strings that contains filenames in order when this funtion returns
*
* returns number of files found
*/

int community_open_and_arrange_dir(char* path, char* suffix, char result[MAX_FILES_IN_DIR][MAX_FILE_NAME_LEN]){
	
	DIR *dir;
	struct dirent *dir_ent;
	int numberOfFiles = 0;
	dir = opendir(path);
	
	if(dir != NULL){
	
		if(strcmp(suffix, "DIR") == 0){
		
			for(numberOfFiles = 0; (dir_ent = readdir(dir)) != 0; numberOfFiles++){
			
				if(strstr(dir_ent->d_name, ".") != 0 || strstr(dir_ent->d_name, "..") != 0)
					numberOfFiles--;

			
				else strcpy(result[numberOfFiles], dir_ent->d_name);

			}
		
		}else{
	
			for(numberOfFiles = 0; (dir_ent = readdir(dir)) != 0; numberOfFiles++){
			
				if(strstr(dir_ent->d_name, suffix) == 0)
					numberOfFiles--;

			
				else	strcpy(result[numberOfFiles], dir_ent->d_name);	

			}
		}
	
		qsort(result, numberOfFiles, MAX_FILE_NAME_LEN, strcoll);
	
		
		closedir(dir);
		
	}

	return numberOfFiles;

}




