/*
 * gems_service_control.c
 *
 * This file is part of JamMo.
 *
 * (c) 2009-2010 University of Oulu, Lappeenranta University of Technology
 *
 * Authors: Tommi Kallonen <tommi.kallonen@lut.fi>
 *          Jussi Laakkonen <jussi.laakkonen@lut.fi>
 */


#include "gems_service_control.h"
#include "gems_message_functions.h"
#include "communication.h"
#include "../cem/cem.h"


gboolean gems_service_control_process_request(gems_connection* element)
{
	gems_components* data = gems_get_data();
	
	gems_teacher_connection* tc = gems_get_teacher_connection();
	
	// Connection to teacher exists, control messages should be coming from teacher only!
	if(tc)
	{
		// Is connected
		if(ph_c_connection_is_connected(tc->connection->connection) == TRUE)
		{
			// Message is coming from teacher
			if(ph_c_connection_get_device_checksum(tc->connection->connection) == ph_c_connection_get_device_checksum(element->connection))
			{	
				switch(gems_connection_get_16(element,6)) // command
				{
					case CONTROL_MODE:
						gems_service_control_handle_control_mode(data,element);
						break;
					case CONTROL_OK:
						gems_service_control_handle_control_ok(data,element);
						break;
					case CONTROL_SET_LOGIN:
						gems_service_control_handle_set_login(data,element);
						break;
					default:
						break;
				}
			}
			else
			{
				cem_add_to_log("gems_service_control_process_request: Got control message from some other than teacher, ignoring.",J_LOG_NETWORK_DEBUG);
				return FALSE;
			}
		}
		else
		{
			cem_add_to_log("gems_service_control_process_request: Got control message but no teacher is connected, ignoring.",J_LOG_NETWORK_DEBUG);
			return FALSE;
		}
	}
	else
	{
		cem_add_to_log("gems_service_control_process_request: Got control message but teacher is not initialized, ignoring.",J_LOG_NETWORK_DEBUG);
		return FALSE;
	}
	
	return TRUE;
}

gems_service_control* gems_service_control_init()
{
	gems_service_control* data = (gems_service_control*)g_malloc(sizeof(gems_service_control));
	
	data->enabled = FALSE;

	data->active=FALSE;
	
	data->port = 0;

	return data;
}

void gems_service_control_cleanup()
{
	gems_service_control* data = gems_get_data()->service_control;
	
	
	if(data != NULL)
	{
		g_free(data);
		data = NULL;
	}
}

void gems_service_control_handle_control_mode(gems_components* data, gems_connection* element)
{
	gchar* logmsg = NULL;
	guint position=8;
	gint16 cntrl_type;
	gint16 lock_mode;

	// Control type
	cntrl_type = gems_connection_get_16(element,position);
	position = position + sizeof(gint16);

	// Lock mode
	lock_mode = gems_connection_get_16(element,position);
	position = position + sizeof(gint16);

	// TODO do something with new control mode

	logmsg = g_strdup_printf("gems_service_control_handle_control_mode: got CONTROL_MODE ");
	cem_add_to_log(logmsg,J_LOG_NETWORK_DEBUG);
	g_free(logmsg);
}

void gems_service_control_handle_control_ok(gems_components* data, gems_connection* element)
{
	gchar* logmsg = NULL;

	logmsg = g_strdup_printf("gems_service_control_handle_control_ok: got CONTROL_OK ");
	cem_add_to_log(logmsg,J_LOG_NETWORK_DEBUG);
	g_free(logmsg);
}

void gems_service_control_handle_set_login(gems_components* data, gems_connection* element)
{
	gchar* logmsg = NULL;
	gint position = sizeof(gint16)+sizeof(gint32)+sizeof(gint16);
	gint length = gems_connection_get_32(element,sizeof(gint16)) - position; // length - header
	
	gchar* loginname = gems_connection_get_data(element,position,length);
	
	logmsg = g_strdup_printf("gems_service_control_handle_set_login: username \"%s\" will be used as login name.",loginname);
	cem_add_to_log(logmsg,J_LOG_NETWORK_DEBUG);
	g_free(logmsg);
	
	// TODO set the loginname!
	
	g_free(loginname);
}
